require 'thread_dumper'

module ThreadDump
  VERSION_MAJOR = 0
  VERSION_MINOR = 0
  VERSION_SUB = 5

  VERSION = "#{VERSION_MAJOR}.#{VERSION_MINOR}.#{VERSION_SUB}"
end

module ThreadDump
  class Config
    class << self
      def dump_target
        @@dump_target ||= :stderr
      end

      def dump_target=(dump_target)
        @@dump_target = dump_target
      end

      def output_format
        @@output_format ||= :text
      end

      def output_format=(output_format)
        @@output_format = output_format
      end
    end
  end
end

module ThreadDump
  class Monitor
    def self.start(time_between_dumps = 60)
      Thread.start(time_between_dumps) do |sleep_time|
        loop do
          ThreadDump::Monitor.write_threads
          sleep sleep_time
        end
      end
    end

    def self.write_threads
      out_arr = []

      ThreadDump::Dumper.dump.each do |thread_info|
        if ThreadDump::Config.output_format == :html
          out_arr << "<dt>#{thread_info[1]}</dt><dd>#{thread_info[0]}</dd>"
        else
          out_arr << "Thread #{thread_info[0]}: #{thread_info[1]}"
        end
      end
      max_l_out_arr = out_arr.map { |out| out.size }.max
      fout = STDERR

      if ThreadDump::Config.dump_target == :file
        fout = File.open("/tmp/#{$$}.ruby_threads.html", "w")
      end

      out_arr.each do |out|
        fout << "-" * max_l_out_arr + "\n" if ThreadDump::Config.output_format == :text
        fout << out + "\n"
      end

      fout << "-" * max_l_out_arr + "\n" if ThreadDump::Config.output_format == :text
      fout.flush

      if fout != STDERR
        fout.close
      end
    end
  end
end

trap("QUIT") do
  Thread.start do
    ThreadDump::Monitor.write_threads
  end
end
